@@ -143,8 +143,13 @@ print_buttons (GsdWacomDevice *device)
 
 		g_print ("\tButton: %s (%s)\n", button->name, button->id);
 		g_print ("\t\tType: %s\n", button_type_to_string (button->type));
-		if (button->group_id > 0)
-			g_print ("\t\tGroup: %d\n", button->group_id);
+		if (button->group_id > 0) {
+			g_print ("\t\tGroup: %d", button->group_id);
+			if (button->idx >= 0)
+				g_print (" Index: %d\n", button->idx);
+			else
+				g_print ("\n");
+		}
 		if (button->settings) {
 			char *loc;
 			loc = get_loc (button->settings);
@@ -190,6 +195,7 @@ list_devices (GList *devices)
 			 gsd_wacom_device_type_to_string (gsd_wacom_device_get_device_type (device)));
 		g_print ("\tReversible: %s\n", BOOL_AS_STR (gsd_wacom_device_reversible (device)));
 		g_print ("\tScreen Tablet: %s\n", BOOL_AS_STR (gsd_wacom_device_is_screen_tablet (device)));
+		g_print ("\tUnknown (fallback) device: %s\n", BOOL_AS_STR(gsd_wacom_device_is_fallback (device)));
 
 		loc = get_loc (gsd_wacom_device_get_settings (device));
 		g_print ("\tGeneric settings: %s\n", loc);
